# Generated by Django 4.2.2 on 2023-06-22 15:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("msgs", "0012_alter_email_object_id_alter_message_object_id_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="email",
            name="service_context",
            field=models.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="message",
            name="service_context",
            field=models.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="sms",
            name="service_context",
            field=models.JSONField(blank=True, null=True),
        ),
    ]



# Generated by Django 3.2.5 on 2022-02-06 11:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('msgs', '0008_auto_20210825_1250'),
    ]

    operations = [
        migrations.CreateModel(
            name='EmailAttachment',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file', models.FileField(upload_to='msgs/attachments/')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AlterField(
            model_name='email',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='emailtemplate',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='message',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='messagetemplate',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='sms',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='smstemplate',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AddField(
            model_name='email',
            name='attachments',
            field=models.ManyToManyField(to='msgs.EmailAttachment'),
        ),
    ]



from typing import Iterable

from django.conf import settings
from django.db import models

from msgs.abstract.models import AbstractTemplate, AbstractMessage, AbstractAttachment
from msgs.helpers import NULLABLE


class SMSTemplate(AbstractTemplate):
    pass


class EmailTemplate(AbstractTemplate):
    pass


class MessageTemplate(AbstractTemplate):
    pass


class EmailAttachment(AbstractAttachment):
    def __str__(self):
        return self.file.url


class SMS(AbstractMessage):
    template = models.ForeignKey(SMSTemplate, on_delete=models.CASCADE)

    def get_provider_name(self):
        return settings.MSGS['sms']

    class Meta:
        verbose_name_plural = 'SMS'


class Email(AbstractMessage):
    template = models.ForeignKey(EmailTemplate, on_delete=models.CASCADE)
    attachments = models.ManyToManyField(EmailAttachment, blank=True)

    reply_to = models.CharField(max_length=256, **NULLABLE)
    cc_emails = models.JSONField(**NULLABLE)
    bcc_emails = models.JSONField(**NULLABLE)

    def get_provider_name(self):
        return settings.MSGS['email']

    def add_attachments(self, attachments: Iterable[str]):
        for item in attachments:
            attachment = EmailAttachment.objects.create(file=item)
            self.attachments.add(attachment)

    def get_cc_emails(self):
        return self.cc_emails or []

    def get_bcc_emails(self):
        return self.bcc_emails or []


class Message(AbstractMessage):
    template = models.ForeignKey(MessageTemplate, on_delete=models.CASCADE)

    def get_provider_name(self):
        return settings.MSGS['message']



from twilio.rest import Client as TwilioClient

from django.conf import settings

from msgs.abstract.models import AbstractMessage

from msgs.providers.base import BaseEmailProvider
from msgs.providers.base import BaseSMSProvider
from msgs.mixins import TemplatingMixin


class TwilioBaseProvider(TemplatingMixin):
    settings = settings.MSGS['providers']['twilio']['options']

    def __init__(self):
        self.client = TwilioClient(
            self.settings['account_sid'],
            self.settings['auth_token'],
        )

    def perform(
            self, message: AbstractMessage, sender: str, lang: str, **kwargs
    ) -> (dict, bool):
        context = self.get_context_data(message)
        _, body = self.render(message, lang, context)

        extra_kwargs = {}
        if sender.startswith('MG'):
            extra_kwargs = {'messaging_service_sid': sender}
        else:
            extra_kwargs = {'from_': sender}

        recipient = message.recipient
        if not recipient.startswith('+'):###################### + is bad for EMAILs!!!!!!!!!!!!!!!!!!
            recipient = '+' + recipient

        result = self.client.messages.create(
            body=body,
            to=recipient,
            **extra_kwargs,
        )
        result = result.__dict__
        message.provider_response = result
        return result, True  # Dummy True

    def process_webhook_event(self, event: dict):
        event_status = event['event']
        timestamp = event['timestamp']
        sg_event_id = event['sg_event_id']
        sg_message_id = event['sg_message_id']



class TwilioEmailProvider(TwilioBaseProvider, BaseEmailProvider):
    pass


class TwilioSMSProvider(TwilioBaseProvider, BaseSMSProvider):
    pass



import base64
import os
import re
import mimetypes

from django.conf import settings
from django.template import Template as DjangoTemplate, Context


class TemplatingMixin(object):
    def build_attachment_object(self, **kwargs):
        """Override it according to your provider's SDK"""
        return

    def get_logo_attachment(self):
        with open(os.path.join(settings.BASE_DIR, 'static', 'logo.png'), 'rb') as f:
            logo = f.read()
        encoded_logo = base64.b64encode(logo).decode()
        return self.build_attachment_object(
            file_content=encoded_logo,
            file_name='logo.png',
            file_type='image/png',
            disposition='inline',
            content_id='logo',
        )

    def get_attachments(self, message, lang, context=None) -> list:
        attachments = []
        if not hasattr(message, 'attachments'):
            return attachments
        for attachment in message.attachments.all():
            encoded_file = base64.b64encode(attachment.file.read()).decode()
            filename = attachment.file.name.rsplit('/', 1)[-1]
            mimetype, _ = mimetypes.guess_type(filename)
            attachments.append(
                self.build_attachment_object(
                    file_content=encoded_file,
                    file_name=filename,
                    file_type=mimetype,
                    disposition='attachment',
                )
            )
        return attachments

    @classmethod
    def html_to_text(cls, html):
        clean_regex = re.compile('<.*?>|&([a-z0-9]+|#[0-9]{1,6}|#x[0-9a-f]{1,6});')
        return re.sub(clean_regex, '', html)

    def render(self, message, lang, context=None):
        title_tpl = DjangoTemplate(f"{message.template.get_subject(lang)}")
        title_html = title_tpl.render(Context(context))
        body_tpl = DjangoTemplate(f"{message.template.get_body(lang)}")
        body_html = body_tpl.render(Context(context))
        return title_html, body_html



import json

from django.conf import settings
from django.core.exceptions import ValidationError
from django.http import HttpResponseBadRequest, HttpResponse
from django.utils.decorators import method_decorator
from django.views import View
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_http_methods

from msgs.helpers import get_provider_class_from_string


@method_decorator(csrf_exempt, name='dispatch')
@method_decorator(require_http_methods(["POST"]), name='dispatch')
class WebhookView(View):
    def post(self, request, provider: str):
        payload = request.body

        provider_settings = settings.MSGS["providers"].get(provider)
        if not provider_settings:
            raise ValidationError("Unknown provider")
        provider = get_provider_class_from_string(provider_settings["backend"])()

        if not provider.check_webhook_signature(request):
            return HttpResponseBadRequest("Invalid signature")

        try:
            events = json.loads(payload)
            provider.webhook(events)
            return HttpResponse("OK", status=200)

        except json.JSONDecodeError:
            return HttpResponseBadRequest("Invalid JSON")
        except Exception as e:
            return HttpResponse("Internal Server Error", status=500)



# Generated by Django 4.2.2 on 2023-06-22 13:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("msgs", "0011_auto_20220613_0857"),
    ]

    operations = [
        migrations.AlterField(
            model_name="email",
            name="object_id",
            field=models.CharField(blank=True, max_length=64, null=True),
        ),
        migrations.AlterField(
            model_name="message",
            name="object_id",
            field=models.CharField(blank=True, max_length=64, null=True),
        ),
        migrations.AlterField(
            model_name="sms",
            name="object_id",
            field=models.CharField(blank=True, max_length=64, null=True),
        ),
    ]



# Generated by Django 3.2.5 on 2021-08-25 12:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('msgs', '0007_auto_20210715_1608'),
    ]

    operations = [
        migrations.AddField(
            model_name='emailtemplate',
            name='external_key',
            field=models.CharField(blank=True, max_length=64, null=True),
        ),
        migrations.AddField(
            model_name='messagetemplate',
            name='external_key',
            field=models.CharField(blank=True, max_length=64, null=True),
        ),
        migrations.AddField(
            model_name='smstemplate',
            name='external_key',
            field=models.CharField(blank=True, max_length=64, null=True),
        ),
        migrations.AlterField(
            model_name='emailtemplate',
            name='key',
            field=models.CharField(max_length=64, unique=True),
        ),
        migrations.AlterField(
            model_name='messagetemplate',
            name='key',
            field=models.CharField(max_length=64, unique=True),
        ),
        migrations.AlterField(
            model_name='smstemplate',
            name='key',
            field=models.CharField(max_length=64, unique=True),
        ),
    ]



from urllib.parse import urlencode

import requests

from collections.abc import Iterable

from django.conf import settings

from msgs.mixins import TemplatingMixin
from msgs.providers.base import BaseMessageProvider
from msgs.abstract.models import AbstractMessage


class TelegramProvider(TemplatingMixin, BaseMessageProvider):
    settings = settings.MSGS['providers']['telegram']['options']
    bot_token = settings['token']

    def get_request_string(self, token, cid, text):
        domain = 'https://api.telegram.org'
        args = urlencode(dict(
            chat_id=cid,
            parse_mode='Markdown',
            text=text,
        ))
        return f'{domain}/bot{token}/sendMessage?{args}'

    def perform(
            self, message: AbstractMessage, sender: str, lang: str, **kwargs
    ) -> (dict, bool):
        context = self.get_context_data(message)
        title_html, body_html = self.render(message, lang, context)
        text = self.get_request_string(self.bot_token, message.recipient, body_html)
        response = requests.get(text)
        return response.json(), True  # Dummy True


class TelegramLoggerProvider(TelegramProvider):
    def get_chat_id(self, user):
        cid = self.settings['chat']
        if not isinstance(cid, Iterable):
            cid = [cid, ]
        return cid

    def perform(
            self, message: AbstractMessage, sender: str, lang: str, **kwargs
    ) -> (dict, bool):
        cids = self.get_chat_id(message.recipient)
        sender = self.get_sender(message)
        context = self.get_context_data(message)
        title_html, body_html = self.render(message, lang, context)
        text = f'from: {sender}\nto:   {message.recipient}\n\ntitle: {title_html}\n\n {body_html}'
        for cid in cids:
            url = self.get_request_string(self.bot_token, cid, text)
            response = requests.get(url)
        return {'status': 'ok'}, True  # Dummy response



"""
Django settings for msgs project.

Generated by 'django-admin startproject' using Django 3.0.7.

For more information on this file, see
https://docs.djangoproject.com/en/3.0/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.0/ref/settings/
"""

import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/3.0/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = ')m0%=k+$mdhtp3w*8^4$cv*sgyy+nhpc%09)-ci1qj0h54!mer'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ["*", ]


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    # 'django_json_widget',

    'msgs',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'config.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'config.wsgi.application'


# Database
# https://docs.djangoproject.com/en/3.0/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}


# Password validation
# https://docs.djangoproject.com/en/3.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/3.0/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.0/howto/static-files/

STATIC_URL = '/static/'


# MSGS
MSGS = {
    'providers': {
        # MY SETTINGS
        'sendgrid': {
            'backend': 'msgs.providers.sendgrid.SendgridEmailProvider',
            'options': {
                'is_active': True,
                'api_key': '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',  # My
                'webhook_verification_key': 'MFkwEwYHKoZIzj0CAQYIKoZIzj0DAQcDQgAEgy47bNSzNjiKSzuOERttvy2G3atoCOOTmFa+Pr29nMd9MsRL7Ym6jnOWfYFLh4NvrL86GGI9vS58HkTgNNRtGQ==',
                'sender': [
                    'sunnywebdevelopment@gmail.com',
                ],
            },
        },
        # # SPEEON SETTINGS AE3 PROD
        # 'sendgrid': {
        #     'backend': 'msgs.providers.sendgrid.SendgridEmailProvider',
        #     'options': {
        #         'is_active': True,
        #         'api_key': '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',
        #         'sender': [
        #             'notifications@hma.ca',
        #             'nicole.f@hma.ca',
        #             'admin@hma.ca',
        #         ],
        #         'skip_duplicates': True,
        #     },
        # },
        'sendinblue': {
            'backend': 'msgs.providers.sendinblue.SendinblueEmailProvider',
            'options': {
                'is_active': True,
                'api_key': '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',
                'sender_email': 'san4ezy@email.com',
                'sender_name': 'test',
            },
        },
        'twilio': {
            'backend': 'msgs.providers.twilio.TwilioProvider',
            'options': {
                'is_active': True,
                'account_sid': 'AC4dbce5d6ef60c86c86561cfc2222ffa9',
                'auth_token': '43030261b80e03860fb0c0752b9262f8',
                'sender': '+13343730227',
            },
        },
        'telegram': {
            'backend': 'msgs.providers.telegram.TelegramProvider',
            'options': {
                'is_active': True,
                'token': '1848056858:AAEfvR26GkporfuAPKhDu0weIEV9dSmlfXw',
                # 'chat': ['352205105', '535989780', ],
                'sender': 'Bot',
            },
        },
        'dummy': {
            'backend': 'msgs.providers.dummy.DummyProvider',
            'options': {
                'is_active': True,
                'model': 'msgs.SMS',
                'sender': 'Dummy',
            }
        },
    },
    'options': {
        'default_language': 'en',
        'skip_duplicates': False,
        'auto_duplicate': True,
        'raise_exceptions': True,
        'auto_delete': False,
    },
    'development': 'telegram',
    'email': 'sendgrid',
    'sms': 'twilio',
    'message': 'telegram',
}



import warnings
from django.conf import settings
from django.apps import apps

from msgs.abstract.models import AbstractMessage
from msgs.providers.base import BaseProvider


class DummyProvider(BaseProvider):
    settings = settings.MSGS['providers']['dummy']['options']

    def __init__(self):
        super().__init__()
        self.model = apps.get_model(self.settings['model'])

    def perform(self, message: AbstractMessage, sender: str, **kwargs) -> (dict, bool):
        warnings.warn(
            f'DummyProvider: {message.recipient}({message.pk}): {message.context}'
        )
        return {}, True



from django.conf import settings

from msgs.exceptions import MSGSProviderIsDisabled, MSGSConfigurationError, \
    MSGSCannotBeSent
from msgs.abstract.models import AbstractMessage
from msgs.models import Email, SMS, Message


class BaseProvider(object):
    client = None
    settings = None
    default_language = 'en'
    model: AbstractMessage = None  # !required!

    def get_lang(self, lang=None):
        if lang:
            return lang
        try:
            return settings.MSGS['options']['default_language']
        except KeyError:
            return self.default_language

    def get_sender(self, message: AbstractMessage, **kwargs) -> dict | str:
        message_sender = message.sender
        settings_sender = self.settings.get('sender')

        if settings_sender is None:
            raise MSGSConfigurationError("Sender must be set.")
        elif settings_sender == '*':
            if not message_sender:
                raise MSGSConfigurationError("Message sender must be set if global option is a `*` symbol.")
            return message_sender
        elif isinstance(settings_sender, str):
            if message_sender:
                raise MSGSConfigurationError("Message sender cannot be set if the global sender used.")
            return settings_sender
        elif isinstance(settings_sender, list) or isinstance(settings_sender, tuple):
            if not message_sender:
                # set first sender as a default if the message sender is not set
                message_sender = settings_sender[0]
            if message_sender not in settings_sender:
                raise MSGSConfigurationError("Message sender not in the allowed list.")
            return message_sender

    @staticmethod
    def get_context_data(message, context=None):
        ctxt = message.context
        if context and isinstance(context, dict):
            ctxt.update(context)
        return ctxt

    def success(self, message: AbstractMessage, response: dict) -> AbstractMessage:
        message.status = self.model.Status.SENT
        message.provider_response = response
        message.provider_id = self.get_provider_id(message, response)
        return message

    def error(self, message: AbstractMessage, error_text: str) -> AbstractMessage:
        message.status = self.model.Status.ERROR
        message.error = error_text
        return message

    def save_message(self, message: AbstractMessage):
        # Call this method for saving message state
        message.save()

    def perform(
            self, message: AbstractMessage, sender: str, lang: str, **kwargs,
    ) -> (dict, bool):
        """
        Override this method according to the particular provider.
        Return:
            response (dict)
            is_success (bool)
        """
        return {}, True

    def pre_send(self, message: AbstractMessage, **kwargs):
        pass

    def post_send(self, message: AbstractMessage, is_success: bool, **kwargs):
        pass

    def get_provider_id(self, message: AbstractMessage, response: dict) -> str:
        pass

    def send(self, message: AbstractMessage, **kwargs) -> bool:
        raise_exceptions = kwargs.get('raise_exceptions', None)
        if not raise_exceptions:
            raise_exceptions = settings.MSGS['options'].get('raise_exceptions', False)
        if message.status != AbstractMessage.Status.IN_QUEUE and raise_exceptions:
            raise MSGSCannotBeSent
        if not kwargs.get('lang'):
            kwargs['lang'] = self.get_lang()
        if not self.settings.get('is_active'):
            raise MSGSProviderIsDisabled(f"{self.__class__.__name__}'s sending is disabled")
        self.pre_send(message, **kwargs)
        response, err = None, None
        try:
            sender = self.get_sender(message, **kwargs)
            response, is_success = self.perform(message, sender, **kwargs)
        except Exception as e:
            self.error(message, str(e))
            err = e
        else:
            self.success(message, response)
            self.post_send(
                message, response=response, is_success=is_success, **kwargs,
            )
        finally:
            self.save_message(message)
            if err and raise_exceptions:
                raise err
            return response

    def process_webhook_event(self, event: dict):
        # Process event data here
        # Must find, update and return message object.
        raise NotImplementedError

    def check_webhook_signature(self, request):
        # Check the payload signature sent with webhook.
        # Implement it on the provider level.
        return True

    def webhook(self, data: list[dict]):
        # Use this method as the API Webhook endpoint and pass the list of events.
        for event in data:
            self.process_webhook_event(event)


class BaseEmailProvider(BaseProvider):
    model = Email


class BaseSMSProvider(BaseProvider):
    model = SMS


class BaseMessageProvider(BaseProvider):
    model = Message


# class ProxyProvider(BaseProvider):
#     proxy_model: BaseProvider
#
#     def send(self, message: AbstractMessage, **kwargs) -> bool:



from typing import Type

from msgs.abstract.models import AbstractMessage


class TemplateContext(dict):
    def __init__(self, key: str, **kwargs):
        super().__init__(**kwargs)
        self.__key = key

    def get_key(self):
        return self.__key


# example = TemplateContext(
#     key='template_unique_key',
#     context_value_1='',
#     context_value_2='',
# )


def build_message(
        model: Type[AbstractMessage],
        recipient: str,
        context: TemplateContext,
) -> AbstractMessage:
    return model(
        recipient=recipient,
        template=model.get_template(
            context.get_key(),
        ),
        context=context,
    )


def send_message(
        model: Type[AbstractMessage],
        recipient: str,
        context: TemplateContext,
        lang: str = None,
) -> AbstractMessage:
    instance = build_message(model, recipient, context)
    instance.send(lang=lang)
    return instance



from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType

from django.conf import settings
from django.db.models import JSONField, Model
from django.db import models
from django.forms import model_to_dict
from django_extensions.db.models import TimeStampedModel
from django.db.models.enums import TextChoices

from django.utils import timezone

from msgs.exceptions import (
    MSGSUnknownProvider, MSGSTemplateDoesNotExist, MSGSSignalNotFound, MSGSCannotBeSent
)
from msgs.helpers import NULLABLE, get_provider_class_from_string
from msgs.signals import (
    SIGNALS, BUILT_SIGNAL, CREATED_SIGNAL, SENT_SIGNAL, STATUS_CHANGED_SIGNAL
)

try:
    from django.utils.translation import gettext_lazy as _
except ImportError:
    from django.utils.translation import ugettext_lazy as _


DEFAULT_TEMPLATE_FIELD = 'template'


class AbstractTemplate(models.Model):
    class Meta:
        abstract = True

    # class Type(TextChoices):
    #     EMAIL = 'email'
    #     SMS = 'sms'
    #     MESSENGER = 'messenger'

    name = models.CharField(max_length=64, default='default name')
    notes = models.CharField(max_length=128, **NULLABLE)

    # type = models.CharField(max_length=16, choices=Type.choices, **NULLABLE)
    key = models.CharField(max_length=64, unique=True)
    external_key = models.CharField(max_length=64, **NULLABLE)

    subject_en = models.CharField(max_length=254, default='Subject')
    body_en = models.TextField()

    def __str__(self):
        return self.name

    def get_body(self, lang):
        return getattr(self, f'body_{lang}')

    def get_subject(self, lang):
        return getattr(self, f'subject_{lang}')


class AbstractMessage(TimeStampedModel):
    class Meta:
        abstract = True

    class Status(TextChoices):
        IN_QUEUE = 'in_queue', _('In Queue')
        SENT = 'sent', _('Sent')
        ERROR = 'error', _('Error')
        DELIVERED = 'delivered', _('Delivered')
        REJECTED = 'rejected', _('Rejected')
        OPENED = 'opened', _('Opened')
        SPAM_REPORTED = 'spam_reported', _('Spam Reported')

    template = models.ForeignKey(AbstractTemplate, on_delete=models.CASCADE)
    recipient = models.CharField(max_length=64)
    sender = models.CharField(max_length=256, **NULLABLE)
    context = JSONField(**NULLABLE)
    status = models.CharField(
        max_length=16, choices=Status.choices, default=Status.IN_QUEUE,
    )
    error = models.TextField(**NULLABLE)

    provider_id = models.CharField(max_length=64, **NULLABLE)
    provider_response = models.TextField(**NULLABLE)

    created_at = models.DateTimeField(auto_now_add=True, **NULLABLE)
    modified_at = models.DateTimeField(auto_now=True, **NULLABLE)
    sent_at = models.DateTimeField(**NULLABLE)

    # additional service information could be saved here
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE, **NULLABLE)
    object_id = models.CharField(max_length=64, **NULLABLE)
    related_to = GenericForeignKey('content_type', 'object_id')
    service_context = JSONField(**NULLABLE)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.__origin_status = self.status

    def __str__(self):
        return f"{self.__class__.__name__} for {self.recipient}"

    def save(self, **kwargs):
        super().save(**kwargs)
        if self.__origin_status != self.status:
            self.send_signal(
                STATUS_CHANGED_SIGNAL,
                instance=self,
                origin_status=self.__origin_status,
            )

    @classmethod
    def build(
            cls,
            template: str,
            recipient: str,
            context: dict,
            related_to: Model = None,
            service_context: dict = None,
            **kwargs,
    ):
        """Returns not saved instance"""
        template = cls.get_template(key=template)
        instance = cls(
            recipient=recipient,
            template=template,
            context=context,
            service_context=service_context,
            **kwargs,
        )
        if related_to:
            instance.related_to = related_to
        cls.send_signal(BUILT_SIGNAL, instance=instance)
        return instance

    @classmethod
    def create(
            cls,
            template: str,
            recipient: str,
            context: dict,
            related_to: Model = None,
            service_context: dict = None,
            skip_duplicates: bool = None,
            **kwargs,
    ):
        if skip_duplicates is None:
            skip_duplicates = settings.MSGS['options'].get('skip_duplicates')
        if skip_duplicates and cls.objects.filter(
                template__key=template,
                status=cls.Status.IN_QUEUE,
                recipient=recipient,
        ).exists():
            # skip duplicates in queue
            return
        instance = cls.build(
            template=template,
            recipient=recipient,
            context=context,
            service_context=service_context,
            related_to=related_to,
            **kwargs,
        )
        instance.save()
        cls.send_signal(CREATED_SIGNAL, instance=instance)
        return instance

    def duplicate(self):
        data = model_to_dict(self, exclude=[
            'id', 'status', 'sent_at', 'service_context', 'provider_response',
            'error', 'provider_id', 'template', 'content_type',
        ])
        data['template'] = self.template
        data['content_type'] = self.content_type
        attachments = []
        if hasattr(self, 'attachments') and 'attachments' in data:
            data.pop('attachments')
            attachments = self.attachments.all()
        duplicated_obj = self._meta.model.objects.create(**data)
        for a in attachments:
            duplicated_obj.attachments.add(a)
        return duplicated_obj

    def get_provider_name(self):
        return 'development'

    def get_provider(self):
        provider = settings.MSGS['providers'].get(self.get_provider_name())
        if not provider:
            raise MSGSUnknownProvider(f"{provider} is unknown")
        provider = provider['backend']
        return get_provider_class_from_string(provider)()

    @classmethod
    def get_template(cls, key: str) -> AbstractTemplate:
        template_model = cls.__get_template_model()
        try:
            template = template_model.objects.get(key=key)
        except template_model.DoesNotExist:
            raise MSGSTemplateDoesNotExist
        else:
            return template

    def get_status(self):
        return [x[1] for x in self.Status.choices if x[0] == self.status][0]

    def send(
            self,
            lang=None,
            raise_exceptions: bool = None,
            duplicate: bool = None,
    ):
        if duplicate is None:
            duplicate = settings.MSGS['options'].get('auto_duplicate', False)
        provider = self.get_provider()
        self.sent_at = timezone.now()
        try:
            provider.send(self, lang=lang, raise_exceptions=raise_exceptions)
        except MSGSCannotBeSent as e:
            if duplicate:  # make a copy of this message and send it
                msg = self.duplicate()
                msg.send(lang=lang)
            else:
                raise e
        else:
            self.send_signal(SENT_SIGNAL, instance=self)

    @classmethod
    def __get_template_model(cls) -> Model:
        return cls._meta.get_field(DEFAULT_TEMPLATE_FIELD).related_model

    @classmethod
    def send_signal(cls, name: str, *args, **kwargs):
        try:
            signal = SIGNALS[cls.__name__][name]
        except KeyError as e:
            raise MSGSSignalNotFound(f'[{cls.__name__}][{name}]')
        else:
            signal.send(cls, *args, **kwargs)


class AbstractAttachment(models.Model):
    class Meta:
        abstract = True

    file = models.FileField(upload_to='msgs/attachments/')



import importlib


NULLABLE = {'blank': True, 'null': True}


def get_provider_class_from_string(reference: str):
    module_name, func = reference.rsplit('.', maxsplit=1)
    module = importlib.import_module(module_name)
    return getattr(module, func)



from msgs.exceptions import MSGSProviderIsDisabled
from msgs.mixins import TemplatingMixin
from msgs.abstract.models import AbstractMessage

import plivo

from msgs.providers.base import BaseSMSProvider

from django.conf import settings


class MSGSPlivoProviderError(Exception):
    pass


class PlivoSMSProvider(TemplatingMixin, BaseSMSProvider):
    settings = settings.MSGS['providers']['plivo']['options']

    def __init__(self):
        if not self.settings.get('is_active'):
            self.client = None
        self.client = plivo.RestClient(
            self.settings['auth_id'],
            self.settings['auth_token'],
        )
        self.callback_url = self.settings['callback_url']
        self.powerpack_uuid = self.settings['powerpack_uuid']
        self.sender = self.settings['sender']
        if not self.powerpack_uuid and not self.sender:
            raise MSGSPlivoProviderError(
                'either one of "powerpack_uuid" or "sender" (source phone number) '
                'must be provided')

    def perform(
            self, message: AbstractMessage, sender: str, lang: str, **kwargs
    ) -> (dict, bool):
        context = self.get_context_data(message)
        _, body = self.render(message, lang, context)
        extra_kwargs = {}
        if self.callback_url:
            extra_kwargs.update({'url': self.callback_url})
        if self.sender:
            extra_kwargs.update({'src': self.sender})
        else:
            extra_kwargs.update({'powerpack_uuid': self.powerpack_uuid})

        response = self.client.messages.create(
            dst=message.recipient,
            text=body,
            **extra_kwargs,
        )
        response = response.__dict__
        message.provider_response = response
        return response, True  # Dummy True

    def get_provider_id(self, message: AbstractMessage, response: dict) -> str:
        return str(response.get('message_uuid', [None, ])[0])



from django.contrib import admin
from django.db.models.fields.json import JSONField

from django_json_widget.widgets import JSONEditorWidget


MESSAGE_FIELDS = ('recipient', 'template', 'status', 'sent_at', 'related_to',)
TEMPLATE_FIELDS = ('name', 'key', 'subject_en',)


class AbstractMessageAdmin(admin.ModelAdmin):
    list_display = MESSAGE_FIELDS
    list_filter = ('status', 'template',)
    actions = ('send', 'duplicate',)
    readonly_fields = ('status', 'created_at', 'modified_at', 'related_to',)
    fieldsets = [
        ('Common Info', {
            'fields': [
                'recipient',
                'sender',
                'template',
                'status',
                'error',
                'related_to',
                'context',
            ],
        }),
        ('Service Info', {
            'fields': [
                'created_at',
                'modified_at',
                'sent_at',
                'service_context',
            ]
        }),
        ('Provider Info', {
            'fields': [
                'provider_id',
                'provider_response',
            ]
        })
    ]

    def send(self, request, queryset):
        for obj in queryset:
            obj.send()
    send.short_description = "Send"

    def duplicate(self, request, queryset):
        for obj in queryset:
            obj.duplicate()
    duplicate.short_description = "Duplicate"

    def has_change_permission(self, request, obj=None):
        return False


class AbstractTemplateAdmin(admin.ModelAdmin):
    list_display = TEMPLATE_FIELDS
    formfield_overrides = {
        JSONField: {'widget': JSONEditorWidget},
    }



# Generated by Django 5.0.6 on 2024-06-04 06:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('msgs', '0015_email_sender_message_sender_sms_sender_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='email',
            name='error',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='message',
            name='error',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='sms',
            name='error',
            field=models.TextField(blank=True, null=True),
        ),
    ]



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('msgs', '0002_timestamped_messages'),
    ]

    operations = [
        migrations.AddField(
            model_name='msg',
            name='provider_id',
            field=models.CharField(max_length=64, null=True, blank=True),
        ),
        migrations.AddField(
            model_name='msg',
            name='provider_response',
            field=models.CharField(max_length=256, null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='msg',
            name='status',
            field=models.PositiveSmallIntegerField(choices=[(0, 'In queue'), (1, 'Sent'), (2, 'Error'), (3, 'Delivered'), (4, 'Rejected'), ], default=0),
        ),
    ]



# Generated by Django 4.2.2 on 2023-10-05 12:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("msgs", "0014_email_bcc_emails_email_cc_emails_email_reply_to_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="email",
            name="sender",
            field=models.CharField(blank=True, max_length=256, null=True),
        ),
        migrations.AddField(
            model_name="message",
            name="sender",
            field=models.CharField(blank=True, max_length=256, null=True),
        ),
        migrations.AddField(
            model_name="sms",
            name="sender",
            field=models.CharField(blank=True, max_length=256, null=True),
        ),
        migrations.AlterField(
            model_name="email",
            name="provider_response",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="email",
            name="reply_to",
            field=models.CharField(blank=True, max_length=256, null=True),
        ),
        migrations.AlterField(
            model_name="message",
            name="provider_response",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="sms",
            name="provider_response",
            field=models.TextField(blank=True, null=True),
        ),
    ]



# Generated by Django 5.0.6 on 2025-07-07 20:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('msgs', '0016_alter_email_error_alter_message_error_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='email',
            name='status',
            field=models.CharField(choices=[('in_queue', 'In Queue'), ('sent', 'Sent'), ('error', 'Error'), ('delivered', 'Delivered'), ('rejected', 'Rejected'), ('opened', 'Opened'), ('spam_reported', 'Spam Reported')], default='in_queue', max_length=16),
        ),
        migrations.AlterField(
            model_name='message',
            name='status',
            field=models.CharField(choices=[('in_queue', 'In Queue'), ('sent', 'Sent'), ('error', 'Error'), ('delivered', 'Delivered'), ('rejected', 'Rejected'), ('opened', 'Opened'), ('spam_reported', 'Spam Reported')], default='in_queue', max_length=16),
        ),
        migrations.AlterField(
            model_name='sms',
            name='status',
            field=models.CharField(choices=[('in_queue', 'In Queue'), ('sent', 'Sent'), ('error', 'Error'), ('delivered', 'Delivered'), ('rejected', 'Rejected'), ('opened', 'Opened'), ('spam_reported', 'Spam Reported')], default='in_queue', max_length=16),
        ),
    ]



from django.conf import settings
from django.db.models.signals import post_save
from django.dispatch import Signal, receiver


BUILT_SIGNAL, CREATED_SIGNAL, SENT_SIGNAL, STATUS_CHANGED_SIGNAL = \
    'built', 'created', 'sent', 'status_changed'


message_built = Signal()
email_built = Signal()
sms_built = Signal()

message_created = Signal()
email_created = Signal()
sms_created = Signal()

message_sent = Signal()
email_sent = Signal()
sms_sent = Signal()

message_status_changed = Signal()
email_status_changed = Signal()
sms_status_changed = Signal()


SIGNALS = {
    'Message': {
        BUILT_SIGNAL: message_built,
        CREATED_SIGNAL: message_created,
        SENT_SIGNAL: message_sent,
        STATUS_CHANGED_SIGNAL: message_status_changed,
    },
    'Email': {
        BUILT_SIGNAL: email_built,
        CREATED_SIGNAL: email_created,
        SENT_SIGNAL: email_sent,
        STATUS_CHANGED_SIGNAL: email_status_changed,
    },
    'SMS': {
        BUILT_SIGNAL: sms_built,
        CREATED_SIGNAL: sms_created,
        SENT_SIGNAL: sms_sent,
        STATUS_CHANGED_SIGNAL: sms_status_changed,
    },
}


@receiver(post_save, sender='msgs.Email')
@receiver(post_save, sender='msgs.SMS')
@receiver(post_save, sender='msgs.Message')
def delete_after_sending(sender, instance, **kwargs):
    auto_delete = settings.MSGS['options'].get('auto_delete', False)
    if auto_delete and instance.status == instance.Status.SENT:
        instance.delete()



# Generated by Django 4.2.2 on 2023-07-30 20:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("msgs", "0013_email_service_context_message_service_context_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="email",
            name="bcc_emails",
            field=models.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="email",
            name="cc_emails",
            field=models.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="email",
            name="reply_to",
            field=models.CharField(blank=True, max_length=64, null=True),
        ),
        migrations.AlterField(
            model_name="email",
            name="attachments",
            field=models.ManyToManyField(blank=True, to="msgs.emailattachment"),
        ),
    ]



from django.conf import settings

from msgs.abstract.models import AbstractMessage

from sendgrid.helpers.eventwebhook import EventWebhook
from sendgrid import SendGridAPIClient, Mail, Email, Attachment, FileContent, FileName, \
    FileType, Disposition, ContentId, CustomArg

from msgs.providers.base import BaseEmailProvider
from msgs.mixins import TemplatingMixin


class SendgridEmailProvider(TemplatingMixin, BaseEmailProvider):
    settings = settings.MSGS['providers']['sendgrid']['options']
    internal_message_id = "internal_message_id"

    def __init__(self):
        self.client = SendGridAPIClient(
            self.settings['api_key']
        )

    def perform(
            self, message: AbstractMessage, sender: str, lang: str, **kwargs
    ) -> (dict, bool):
        context = self.get_context_data(message)
        title_html, body_html = self.render(message, lang, context)
        attachments = self.get_attachments(message, lang, context)
        sendgrid_message = Mail(
            from_email=self.get_sender(message),
            to_emails=message.recipient,
            subject=title_html,
            html_content=body_html,
        )

        if message.reply_to:
            sendgrid_message.reply_to = message.reply_to

        for cc in message.get_cc_emails():
            sendgrid_message.personalizations[0].add_cc(Email(cc))

        for bcc in message.get_bcc_emails():
            sendgrid_message.personalizations[0].add_bcc(Email(bcc))

        for attachment in attachments:
            sendgrid_message.add_attachment(attachment)
        # sendgrid_message.add_attachment(self.get_logo_attachment())  # must be removed to the child class for the library version

        sendgrid_message.custom_arg = CustomArg(
            self.internal_message_id,
            str(message.id),
        )

        response = self.client.send(sendgrid_message)
        response_data = dict(
            status_code=response.status_code,
            body=response.body,
            headers=response.headers,
        )

        return response_data, 200 <= response.status_code < 300

    def build_attachment_object(self, **kwargs):
        attachment_kwargs = [
            FileContent(kwargs['file_content']),
            FileName(kwargs['file_name']),
            FileType(kwargs['file_type']),
            Disposition(kwargs['disposition']),
        ]
        if 'content_id' in kwargs:
            attachment_kwargs.append(ContentId(kwargs['content_id']))
        return Attachment(*attachment_kwargs)

    def check_webhook_signature(self, request):
        public_key = self.settings.get("webhook_verification_key")

        event_webhook = EventWebhook()
        ec_public_key = event_webhook.convert_public_key_to_ecdsa(public_key)

        return event_webhook.verify_signature(
            request.body.decode(),
            request.META['HTTP_X_TWILIO_EMAIL_EVENT_WEBHOOK_SIGNATURE'],
            request.META['HTTP_X_TWILIO_EMAIL_EVENT_WEBHOOK_TIMESTAMP'],
            ec_public_key,
        )

    def process_webhook_event(self, event: dict):
        message_id = event.get(self.internal_message_id)
        if not message_id:
            return None

        event_status = {
            "opened": AbstractMessage.Status.OPENED,
            "delivered": AbstractMessage.Status.DELIVERED,
            "dropped": AbstractMessage.Status.REJECTED,
            "deferred": AbstractMessage.Status.REJECTED,
            "bounced": AbstractMessage.Status.REJECTED,
            "spamreport": AbstractMessage.Status.SPAM_REPORTED,
        }.get(event.get("event"))
        if event_status:
            self.model.objects.filter(id=message_id).update(status=event_status)
        return None



class VoximplantProvider(Provider):
    settings = settings.VOXIMPLANT

    def __init__(self):
        self.client = VoximplantAPI(self.settings['credentials_path'])

    def get_sender(self, message: AbstractMessage, **kwargs):
        return super().get_sender(message).strip('+')

    def perform(
            self, message: Msg, sender: str, lang: str, **kwargs
    ) -> (dict, bool):
        try:
            return self.client.send_sms_message(
                sender,
                message.recipient.phone_number.strip('+'),
                f"{message.text}",
            ), True  # Dummy True
        except VoximplantException as e:
            print(f"Error: {e.message}")



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('msgs', '0004_default_templates_datamigration'),
    ]

    operations = [
        migrations.AddField(
            model_name='tpl',
            name='name',
            field=models.CharField(max_length=64, default='default_name'),
        ),
        migrations.AddField(
            model_name='tpl',
            name='notes',
            field=models.CharField(max_length=128, blank=True, null=True),
        ),
        migrations.AddField(
            model_name='tpl',
            name='type',
            field=models.CharField(max_length=16, choices=[('email', 'email'), ('sms', 'sms'), ('messenger', 'messenger')], blank=True, null=True),
        ),
    ]



from django.db import migrations, models


DEFAULT_KEY = 'default'


def create_default_template(apps, schema_editor):
    model = apps.get_model('msgs', 'Tpl')
    if model.objects.count() > 0:
        return  # do nothing if at least one template exists
    model.objects.get_or_create(
        key=DEFAULT_KEY,
        subject_en='Default',
        body_en='Hello! This is a default template.',
    )


def delete_default_template(apps, schema_editor):
    model = apps.get_model('msgs', 'Tpl')
    # delete the default template only
    if model.objects.count() == 1 and model.objects.last().key == DEFAULT_KEY:
        model.objects.last().delete()


class Migration(migrations.Migration):

    dependencies = [
        ('msgs', '0003_provider_fields_and_statuses'),
    ]

    operations = [
        migrations.RunPython(create_default_template, delete_default_template)
    ]



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('msgs', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='msg',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, null=True, blank=True),
        ),
        migrations.AddField(
            model_name='msg',
            name='modified_at',
            field=models.DateTimeField(auto_now=True, null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='msg',
            name='context',
            field=models.JSONField(blank=True, null=True),
        ),
    ]



from django.contrib import admin


from msgs.abstract.admin import AbstractMessageAdmin, AbstractTemplateAdmin
from msgs.models import (
    SMSTemplate, EmailTemplate, MessageTemplate, SMS, Email, Message, EmailAttachment,
)


@admin.register(SMSTemplate)
class SMSTemplateAdmin(AbstractTemplateAdmin):
    pass


@admin.register(EmailTemplate)
class EmailTemplateAdmin(AbstractTemplateAdmin):
    pass


@admin.register(MessageTemplate)
class MessageTemplateAdmin(AbstractTemplateAdmin):
    pass


@admin.register(Message)
class MessageAdmin(AbstractMessageAdmin):
    pass


@admin.register(Email)
class EmailAdmin(AbstractMessageAdmin):
    fieldsets = [
        ('Common Info', {
            'fields': [
                'recipient',
                'sender',
                'reply_to',
                'cc_emails',
                'bcc_emails',
                'template',
                'status',
                'error',
                'related_to',
                'context',
            ],
        }),
        ('Service Info', {
            'fields': [
                'created_at',
                'modified_at',
                'sent_at',
                'service_context',
            ]
        }),
        ('Provider Info', {
            'fields': [
                'provider_id',
                'provider_response',
            ]
        })
    ]


@admin.register(SMS)
class SMSAdmin(AbstractMessageAdmin):
    pass


@admin.register(EmailAttachment)
class EmailAttachmentAdmin(admin.ModelAdmin):
    pass



from django.conf import settings
from msgs.mixins import TemplatingMixin

from pysendpulse.pysendpulse import PySendPulse

from msgs.abstract.models import AbstractMessage
from msgs.providers.base import BaseEmailProvider


class SendpulseEmailProvider(TemplatingMixin, BaseEmailProvider):
    settings = settings.MSGS['providers']['sendpulse']['options']

    def __init__(self):
        self.client = PySendPulse(
            self.settings['api_id'],
            self.settings['api_secret'],
            # TOKEN_STORAGE,
            # memcached_host=MEMCACHED_HOST,
        )

    def perform(
            self, message: AbstractMessage, sender: str, lang: str, **kwargs
    ) -> (dict, bool):
        context = self.get_context_data(message)
        title_html, body_html = self.render(message, lang, context)
        # attachments = self.get_attachments(message, lang, context)
        message_data = {
            'subject': title_html,
            'html': body_html,
            'text': self.html_to_text(body_html),
            'from': {
                'name': self.settings['sender_name'],
                'email': self.settings['sender'],
            },
            'to': [
                {
                    'name': '',
                    'email': message.recipient,
                },
            ],
        }
        response = self.client.smtp_send_mail(message_data)
        return response.json(), True  # Dummy True


class SendpulseTemplatingEmailProvider(SendpulseEmailProvider):
    def perform(
            self, message: AbstractMessage, sender: str, lang: str, **kwargs
    ) -> (dict, bool):
        context = self.get_context_data(message)
        title_html, body_html = self.render(message, lang, context)
        message_data = {
            'subject': title_html,
            'from': {
                'name': self.settings['sender_name'],
                'email': self.settings['sender'],
            },
            'to': [
                {
                    'name': '',
                    'email': message.recipient,
                },
            ],
            "template": {
                'id': message.template.external_key,
                'variables': context,
            },
        }
        response = self.client.smtp_send_mail_with_template(message_data)
        return response.json(), True  # Dummy True



# Generated by Django 4.0.4 on 2022-06-13 08:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('msgs', '0010_auto_20220328_1109'),
    ]

    operations = [
        migrations.AlterField(
            model_name='email',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='emailattachment',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='emailtemplate',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='message',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='messagetemplate',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='sms',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='smstemplate',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('msgs', '0005_tpl_name_and_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='tpl',
            name='name',
            field=models.CharField(default='default name', max_length=64),
        ),
        migrations.AlterField(
            model_name='tpl',
            name='type',
            field=models.CharField(blank=True, choices=[('email', 'Email'), ('sms', 'Sms'), ('messenger', 'Messenger')], max_length=16, null=True),
        ),
    ]


# Generated by Django 3.1.3 on 2020-12-01 15:39

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion
import django_extensions.db.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Tpl',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(max_length=32, unique=True)),
                ('subject_en', models.CharField(default='Subject', max_length=254)),
                ('body_en', models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name='Msg',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('recipient', models.CharField(max_length=64)),
                ('context', django.contrib.postgres.fields.jsonb.JSONField(blank=True, null=True)),
                ('status', models.PositiveSmallIntegerField(choices=[(0, 'In queue'), (1, 'Sent'), (2, 'Error')], default=0)),
                ('error', models.CharField(blank=True, max_length=256, null=True)),
                ('tpl', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='msgs.tpl')),
            ],
            options={
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Email',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('msgs.msg',),
        ),
        migrations.CreateModel(
            name='SMS',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('msgs.msg',),
        ),
    ]



# Generated by Django 3.2.5 on 2022-03-28 11:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('msgs', '0009_auto_20220206_1105'),
    ]

    operations = [
        migrations.AddField(
            model_name='email',
            name='sent_at',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='message',
            name='sent_at',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='sms',
            name='sent_at',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]



import json
import requests

from django.conf import settings

from msgs.providers.base import BaseEmailProvider
from msgs.mixins import TemplatingMixin
from msgs.abstract.models import AbstractMessage


class SendinblueEmailProvider(TemplatingMixin, BaseEmailProvider):
    settings = settings.MSGS['providers']['sendinblue']['options']

    def __init__(self):
        self.url = "https://api.sendinblue.com/v3/smtp/email"

    def get_sender(self, message: AbstractMessage, **kwargs) -> dict:
        return {
            'email': self.settings['sender_email'],
            'name': self.settings['sender_name'],
        }

    def perform(
            self, message: AbstractMessage, sender: str, lang: str, **kwargs
    ) -> (dict, bool):
        context = self.get_context_data(message)
        title_html, body_html = self.render(message, lang, context)
        attachments = self.get_attachments(message, lang, context)

        headers = {
            "accept": "application/json",
            "content-type": "application/json",
            "api-key": self.settings['api_key'],
        }

        data = {
            'sender': self.get_sender(message),
            'htmlContent': body_html,
            'subject': title_html,
            'to': [{'name': message.recipient, 'email': message.recipient}, ],
            'params': {
                'LOGO': 'https://example.com/logo.png',
            },
        }
        if attachments:
            data.update({'attachment': attachments})

        response = requests.request("POST", self.url, headers=headers, data=json.dumps(data))
        return response.json(), True  # Dummy True

    def build_attachment_object(self, **kwargs):
        return {
            'content': kwargs['file_content'],
            'name': kwargs['file_name'],
        }



class MSGSUnknownProvider(Exception):
    pass


class MSGSProviderIsDisabled(Exception):
    pass


class MSGSTemplateDoesNotExist(Exception):
    pass


class MSGSSignalNotFound(Exception):
    pass


class MSGSConfigurationError(Exception):
    pass


class MSGSCannotBeSent(Exception):
    def __init__(self):
        self.message = (
            "Message instance cannot be sent twice. "
            "Duplicate it instead of multiple sending "
            "or set the `auto_duplicate` option."
        )
        super().__init__(self.message)



import sib_api_v3_sdk
from sib_api_v3_sdk.rest import ApiException

from django.conf import settings

from msgs.providers.base import BaseProvider
from msgs.mixins import TemplatingMixin


class SendinblueSDKProvider(TemplatingMixin, BaseProvider):
    settings = settings.MSGS['providers']['sendinblue_sdk']['options']

    def __init__(self):
        configuration = sib_api_v3_sdk.Configuration()
        configuration.api_key['api-key'] = self.settings['api_key']
        self.configuration = configuration
        self.client = sib_api_v3_sdk.SendSmtpEmail

    @property
    def sender_name(self):
        return self.settings['sender_name']

    @property
    def sender_email(self):
        return self.settings['sender_email']

    def get_sender(self, message: AbstractMessage, **kwargs) -> str:
        return self.sender_email

    def perform(
            self, message: AbstractMessage, sender: str, lang: str, **kwargs
    ) -> (dict, bool):
        title_html, body_html, attachments = self.render(message, lang)
        api_instance = sib_api_v3_sdk.SMTPApi(sib_api_v3_sdk.ApiClient(self.configuration))
        senderSmtp = sib_api_v3_sdk.SendSmtpEmailSender(
            name=self.sender_name,
            email=self.sender_email,
        )
        sendTo = sib_api_v3_sdk.SendSmtpEmailTo(
            email=message.recipient,
            name="Recipient Name"  # what's the name?
        )
        arrTo = [sendTo]
        sending_kwargs = {
            "sender": senderSmtp,
            "to": arrTo,
            "html_content": body_html,
            "subject": title_html,
        }
        # attachments.append(self.get_logo_attachment())
        if attachments:
            sending_kwargs['attachment'] = attachments
        send_smtp_email = sib_api_v3_sdk.SendSmtpEmail(**sending_kwargs)
        api_response = api_instance.send_transac_email(send_smtp_email)
        return api_response.to_dict, True  # Dummy True

    def build_attachment_object(self, **kwargs):
        return sib_api_v3_sdk.SendSmtpEmailAttachment(
            content=kwargs['file_content'],
            name=kwargs['file_name'],
        )



# Generated by Django 3.2.5 on 2021-07-15 16:08

from django.db import migrations, models
import django.db.models.deletion
import django_extensions.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('msgs', '0006_tpl_fields_changed'),
    ]

    operations = [
        migrations.CreateModel(
            name='EmailTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(default='default name', max_length=64)),
                ('notes', models.CharField(blank=True, max_length=128, null=True)),
                ('key', models.CharField(max_length=32, unique=True)),
                ('subject_en', models.CharField(default='Subject', max_length=254)),
                ('body_en', models.TextField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('recipient', models.CharField(max_length=64)),
                ('context', models.JSONField(blank=True, null=True)),
                ('status', models.CharField(choices=[('in_queue', 'In queue'), ('sent', 'Sent'), ('error', 'Error'), ('delivered', 'Delivered'), ('rejected', 'Rejected')], default='in_queue', max_length=16)),
                ('error', models.CharField(blank=True, max_length=256, null=True)),
                ('provider_id', models.CharField(blank=True, max_length=64, null=True)),
                ('provider_response', models.CharField(blank=True, max_length=256, null=True)),
                ('created_at', models.DateTimeField(auto_now_add=True, null=True)),
                ('modified_at', models.DateTimeField(auto_now=True, null=True)),
                ('object_id', models.PositiveIntegerField(blank=True, null=True)),
                ('content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MessageTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(default='default name', max_length=64)),
                ('notes', models.CharField(blank=True, max_length=128, null=True)),
                ('key', models.CharField(max_length=32, unique=True)),
                ('subject_en', models.CharField(default='Subject', max_length=254)),
                ('body_en', models.TextField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SMSTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(default='default name', max_length=64)),
                ('notes', models.CharField(blank=True, max_length=128, null=True)),
                ('key', models.CharField(max_length=32, unique=True)),
                ('subject_en', models.CharField(default='Subject', max_length=254)),
                ('body_en', models.TextField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.RemoveField(
            model_name='msg',
            name='tpl',
        ),
        migrations.DeleteModel(
            name='Email',
        ),
        migrations.DeleteModel(
            name='SMS',
        ),
        migrations.CreateModel(
            name='Email',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('recipient', models.CharField(max_length=64)),
                ('context', models.JSONField(blank=True, null=True)),
                ('status', models.CharField(choices=[('in_queue', 'In queue'), ('sent', 'Sent'), ('error', 'Error'), ('delivered', 'Delivered'), ('rejected', 'Rejected')], default='in_queue', max_length=16)),
                ('error', models.CharField(blank=True, max_length=256, null=True)),
                ('provider_id', models.CharField(blank=True, max_length=64, null=True)),
                ('provider_response', models.CharField(blank=True, max_length=256, null=True)),
                ('created_at', models.DateTimeField(auto_now_add=True, null=True)),
                ('modified_at', models.DateTimeField(auto_now=True, null=True)),
                ('object_id', models.PositiveIntegerField(blank=True, null=True)),
                ('content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
                ('template', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='msgs.emailtemplate')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SMS',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('recipient', models.CharField(max_length=64)),
                ('context', models.JSONField(blank=True, null=True)),
                ('status', models.CharField(choices=[('in_queue', 'In queue'), ('sent', 'Sent'), ('error', 'Error'), ('delivered', 'Delivered'), ('rejected', 'Rejected')], default='in_queue', max_length=16)),
                ('error', models.CharField(blank=True, max_length=256, null=True)),
                ('provider_id', models.CharField(blank=True, max_length=64, null=True)),
                ('provider_response', models.CharField(blank=True, max_length=256, null=True)),
                ('created_at', models.DateTimeField(auto_now_add=True, null=True)),
                ('modified_at', models.DateTimeField(auto_now=True, null=True)),
                ('object_id', models.PositiveIntegerField(blank=True, null=True)),
                ('content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
                ('template', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='msgs.smstemplate')),
            ],
            options={
                'verbose_name_plural': 'SMS',
            },
        ),
        migrations.DeleteModel(
            name='Msg',
        ),
        migrations.DeleteModel(
            name='Tpl',
        ),
        migrations.AddField(
            model_name='message',
            name='template',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='msgs.messagetemplate'),
        ),
    ]
